// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/bbtest/auto_trans.c $
// $Rev: 51802 $
// $Date: 2022-10-14 12:32:06 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



// #define	********************************************************************

#define	CHECK_LIMIT		1000
#define	TEST_LIMIT		1000



//*****************************************************************************
static void _show_results(
	int					errs,
	long				max,
	const long*			max_0,
	const long*			max_1,
	const long long*	sums_0,
	const long long*	sums_1)
{
	char	buf[64];
	int		i;

	gsc_label_level_inc();

	if ((errs == 0) && (max == 1))
	{
		gsc_label("All 32 Ports");
		printf("All in one read cycle.\n");
	}
	else
	{
		for (i = 0; i <= 31; i++)
		{
			sprintf(buf, "Port %2d", i);
			gsc_label(buf);

			if ((sums_1[i] < 0) || (sums_1[i] < 0) || (max_0[i] < 0) || (max_1[i] < 0))
			{
				printf("ERROR <---\n");
			}
			else if ((max_0[i] == 1) && (max_1[i] == 1))
			{
				printf("All in one read cycle.\n");
			}
			else
			{
				printf(	"0->1: avg = %.3f, max = %ld, 1->0: avg = %.3f, max = %ld\n",
						(double) sums_1[i] / TEST_LIMIT,
						max_1[i],
						(double) sums_0[i] / TEST_LIMIT,
						max_0[i]);
			}
		}
	}

	gsc_label_level_dec();
}



//*****************************************************************************
static int _tx_vout_to_rx_vout(int index_1, int fd_1, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout Readback (#%d)", index_1);
	gsc_label(buf);
	printf("(write Vout once, read Vout as needed)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	errs	+= dio32_initialize(fd_2, -1, 0);
	errs	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

	errs	+= dio32_initialize(fd_1, -1, 0);
	errs	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_OVR, &get);

				if ((get == set) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_OVR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx_vout_to_rx_vin(int index_1, int fd_1, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout to Vin (#%d)", index_1);
	gsc_label(buf);
	printf("(write Vout once, read Vin as needed)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	errs	+= dio32_initialize(fd_2, -1, 0);
	errs	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

	errs	+= dio32_initialize(fd_1, -1, 0);
	errs	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == set) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx1_vout_to_rx2_vin(int index_1, int fd_1, int index_2, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout #%d to Vin #%d", index_1, index_2);
	gsc_label(buf);
	printf("(write Vout once, read Vin as needed)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	// Initialize both devices/
	errs	+= dio32_initialize(fd_1, -1, 0);
	errs	+= dio32_initialize(fd_2, -1, 0);

	// Configure the first device as all outputs.
	// Leave the second device as all inputs.
	errs	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == set) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx1_vout_to_rx2_vout_same(int index_1, int fd_1, int index_2, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout #%d to Vout #%d (same)", index_1, index_2);
	gsc_label(buf);
	printf("(write Vout once, read transitioned Vout as needed, LH->LH, HL->HL)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		// Initialize both devices.
		err	+= dio32_initialize(fd_1, -1, 0);
		err	+= dio32_initialize(fd_2, -1, 0);

		// Configure the first device with one output.
		// Configure the second device with one input.
		err	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		err	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, ~set, NULL);

		// Configure the second device so the outputs transition to the same level as the input.
		err	+= dio32_hl_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
		err	+= dio32_lh_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_hl_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

		// Begin testing.

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OVR, &get);

				if ((get == ~set) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OVR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx1_vout_to_rx2_vout_diff(int index_1, int fd_1, int index_2, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout #%d to Vout #%d (diff)", index_1, index_2);
	gsc_label(buf);
	printf("(write Vout once, read transitioned Vout as needed, LH->LH, HL->LH)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		// Initialize both devices.
		err	+= dio32_initialize(fd_1, -1, 0);
		err	+= dio32_initialize(fd_2, -1, 0);

		// Configure the first device with one output.
		// Configure the second device with one input.
		err	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		err	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, ~set, NULL);

		// Configure the second device so the outputs transition to the alternate level as the input.
		err	+= dio32_hl_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
		err	+= dio32_hl_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

		// Begin testing.

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OVR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OVR, &get);

				if ((get == ~set) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx1_vout_to_2_rx1_vin_same(int index_1, int fd_1, int index_2, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout #%d to #%d to Vin #%d (same)", index_1, index_2, index_1);
	gsc_label(buf);
	printf("(write Vout once, read transitioned Vin as needed, LH->LH, HL->HL)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		// Initialize both devices.
		err	+= dio32_initialize(fd_1, -1, 0);
		err	+= dio32_initialize(fd_2, -1, 0);

		// Configure the first device with one output.
		// Configure the second device with one input.
		err	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		err	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, ~set, NULL);

		// Configure the second device so the outputs transition to the same level as the input.
		err	+= dio32_hl_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
		err	+= dio32_lh_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_hl_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

		// Begin testing.

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == 0xFFFFFFFF) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx1_vout_to_2_rx1_vin_diff(int index_1, int fd_1, int index_2, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout #%d to #%d to Vin #%d (diff)", index_1, index_2, index_1);
	gsc_label(buf);
	printf("(write Vout once, read transitioned Vin as needed, LH->HL, HL->LH)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		// Initialize both devices.
		err	+= dio32_initialize(fd_1, -1, 0);
		err	+= dio32_initialize(fd_2, -1, 0);

		// Configure the first device with one output.
		// Configure the second device with one input.
		err	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		err	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, ~set, NULL);

		// Configure the second device so the outputs transition to the same level as the input.
		err	+= dio32_hl_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_value	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
		err	+= dio32_hl_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_mask	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

		// Begin testing.

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == set) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_1, -1, 0, DIO32_GSC_PIR, &get);

				if ((get == ~set) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx1_vout_to_2_rx1_vout_ts_same(int index_1, int fd_1, int index_2, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout #%d to Vout TriState #%d (same)", index_1, index_2);
	gsc_label(buf);
	printf("(write Vout once, read transitioned Vout TriState as needed, LH->TriState, HL->Drive)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		// Initialize both devices.
		err	+= dio32_initialize(fd_1, -1, 0);
		err	+= dio32_initialize(fd_2, -1, 0);

		// Configure the first device with one output.
		// Configure the second device with one input.
		err	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		err	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, ~set, NULL);

		// Configure the second device so the outputs transition to the same level as the input.
		err	+= dio32_hl_out_tri_state	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
		err	+= dio32_lh_out_tri_state	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_hl_out_mask		(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_mask		(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

		// Begin testing.

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OTSR, &get);

				if ((get == ~set) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OTSR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
static int _tx1_vout_to_2_rx1_vout_ts_diff(int index_1, int fd_1, int index_2, int fd_2)
{
	char		buf[64];
	int			check;
	int			count;
	int			err;
	int			errs	= 0;
	u32			get;
	long		max		= 0;
	long		max_0[32];
	long		max_1[32];
	int			port;
	u32			set;
	long long	sums_0[32];
	long long	sums_1[32];

	sprintf(buf, "Vout #%d to Vout TriState #%d (diff)", index_1, index_2);
	gsc_label(buf);
	printf("(write Vout once, read transitioned Vout TriState as needed, LH->Drive, HL->TriState)\n");

	memset(max_0, 0, sizeof(max_0));
	memset(max_1, 0, sizeof(max_1));
	memset(sums_0, 0, sizeof(sums_0));
	memset(sums_1, 0, sizeof(sums_1));

	for (port = 0; port <= 31; port++)
	{
		err	= 0;
		set	= 1 << port;

		// Initialize both devices.
		err	+= dio32_initialize(fd_1, -1, 0);
		err	+= dio32_initialize(fd_2, -1, 0);

		// Configure the first device with one output.
		// Configure the second device with one input.
		err	+= dio32_io_out_dir(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
		err	+= dio32_io_out_dir(fd_2, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, ~set, NULL);

		// Configure the second device so the outputs transition to the same level as the input.
		err	+= dio32_hl_out_tri_state	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_tri_state	(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
		err	+= dio32_hl_out_mask		(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
		err	+= dio32_lh_out_mask		(fd_2, -1, 0, port, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);

		// Begin testing.

		for (count = 1; count <= TEST_LIMIT; count++)
		{
			// Check for the 0->1 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OTSR, &get);

				if ((get == 0) || (err))
					break;
			}

			if (err)
				break;

			sums_1[port]	+= check;

			if (check > max_1[port])
				max_1[port]	= check;

			if (check > max)
				max	= check;

			// Check for the 1->0 changes.
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, set, NULL);
			err	+= dio32_io_out_val(fd_1, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

			for (check = 1; check <= CHECK_LIMIT; check++)
			{
				err	+= dio32_reg_read(fd_2, -1, 0, DIO32_GSC_OTSR, &get);

				if ((get == ~set) || (err))
					break;
			}

			if (err)
				break;

			sums_0[port]	+= check;

			if (check > max_0[port])
				max_0[port]	= check;

			if (check > max)
				max	= check;
		}

		if (err)
		{
			max_0[port]		= -1;
			max_1[port]		= -1;
			sums_0[port]	= -1;
			sums_1[port]	= -1;
		}

		errs	= err ? 1 : 0;
	}

	// Report the results.
	_show_results(errs, max, max_0, max_1, sums_0, sums_1);

	return(errs);
}



//*****************************************************************************
int auto_trans_tests(int index_1, int fd_1, int index_2, int fd_2)
{
	char	buf[64];
	int		errs	= 0;

	gsc_label("Auto-Transition Tests");
	printf("(%d iterations: apply a change, read to validate results)\n", TEST_LIMIT);
	gsc_label_level_inc();

	if (index_1 == index_2)
	{
		errs	+= _tx_vout_to_rx_vout	(index_1, fd_1, fd_2);
		errs	+= _tx_vout_to_rx_vin	(index_1, fd_1, fd_2);
	}
	else
	{
		sprintf(buf, "#%d -> #%d", index_1, index_2);
		gsc_label(buf);
		printf("\n");

		gsc_label_level_inc();
		errs	+= _tx_vout_to_rx_vout				(index_1, fd_1, fd_2);
		errs	+= _tx_vout_to_rx_vin				(index_1, fd_1, fd_2);
		errs	+= _tx1_vout_to_rx2_vin				(index_1, fd_1, index_2, fd_2);
		errs	+= _tx1_vout_to_rx2_vout_same		(index_1, fd_1, index_2, fd_2);
		errs	+= _tx1_vout_to_rx2_vout_diff		(index_1, fd_1, index_2, fd_2);
		errs	+= _tx1_vout_to_2_rx1_vin_same		(index_1, fd_1, index_2, fd_2);
		errs	+= _tx1_vout_to_2_rx1_vin_diff		(index_1, fd_1, index_2, fd_2);
		errs	+= _tx1_vout_to_2_rx1_vout_ts_same	(index_1, fd_1, index_2, fd_2);
		errs	+= _tx1_vout_to_2_rx1_vout_ts_diff	(index_1, fd_1, index_2, fd_2);
		gsc_label_level_dec();

		sprintf(buf, "#%d -> #%d", index_2, index_1);
		gsc_label(buf);
		printf("\n");

		gsc_label_level_inc();
		errs	+= _tx_vout_to_rx_vout				(index_1, fd_1, fd_2);
		errs	+= _tx_vout_to_rx_vin				(index_1, fd_1, fd_2);
		errs	+= _tx1_vout_to_rx2_vin				(index_2, fd_2, index_1, fd_1);
		errs	+= _tx1_vout_to_rx2_vout_same		(index_2, fd_2, index_1, fd_1);
		errs	+= _tx1_vout_to_rx2_vout_diff		(index_2, fd_2, index_1, fd_1);
		errs	+= _tx1_vout_to_2_rx1_vin_same		(index_2, fd_2, index_1, fd_1);
		errs	+= _tx1_vout_to_2_rx1_vin_diff		(index_2, fd_2, index_1, fd_1);
		errs	+= _tx1_vout_to_2_rx1_vout_ts_same	(index_2, fd_2, index_1, fd_1);
		errs	+= _tx1_vout_to_2_rx1_vout_ts_diff	(index_2, fd_2, index_1, fd_1);
		gsc_label_level_dec();
	}

	gsc_label_level_dec();
	return(errs);
}


