// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/bbtest/main.c $
// $Rev: 51802 $
// $Date: 2022-10-14 12:32:06 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	int		_continuous		= 0;
static	int		_def_index_1	= 0;
static	int		_def_index_2	= 0;
static	int		_ignore_errors	= 0;
static	long	_minute_limit	= 0;
static	int		_test_limit		= -1;



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	char	buf[64];
	char	c;
	int		errs	= 0;
	int		first	= 0;
	int		i;
	int		j;
	int		k;
	int		second	= 0;

	printf("bbtest - Board-to-Board test\n");
	printf("USAGE: bbtest <-c> <-C> <-m#> <-n#> <index 1> <index 2>\n");
	printf("  -c       Continue testing until an error occurs.\n");
	printf("  -C       Continue testing even if errors occur.\n");
	printf("  -m#      Run for at most # minutes (a decimal number).\n");
	printf("  -n#      Repeat test at most # times (a decimal number).\n");
	printf("  index 1  The zero based index of the first device to access.\n");
	printf("  index 2  The zero based index of the second device to access.\n");

	gsc_label("Arguments");
	printf("(%d argument%s)\n", argc - 1, ((argc - 1) == 1) ? "s" : "");
	gsc_label_level_inc();

	for (i = 0; i < argc; i++)
	{
		sprintf(buf, "Argument %d", i);
		gsc_label(buf);
		printf("%s\n", argv[i]);
	}

	gsc_label_level_dec();

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-c") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 0;
			continue;
		}

		if (strcmp(argv[i], "-C") == 0)
		{
			_continuous		= 1;
			_ignore_errors	= 1;
			continue;
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'm') &&
			(argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_minute_limit	= k;
				continue;
			}
		}

		if ((argv[i][0] == '-') &&
			(argv[i][1] == 'n') &&
			(argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_test_limit	= k;
				continue;
			}
		}

		j	= sscanf(argv[i], "%d%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			if (first == 0)
			{
				first			= 1;
				_def_index_1	= k;
				continue;
			}

			if (second == 0)
			{
				second			= 1;
				_def_index_2	= k;
				continue;
			}
		}

		errs	= 1;
		printf("ERROR: invalid argument: #%d = '%s'\n", i, argv[i]);
		break;
	}

	return(errs);
}



//*****************************************************************************
static void _show_access_index(int index_1, int index_2)
{
	gsc_label("Accessing Device");

	if (index_1 == index_2)
		printf("%d\n", index_1);
	else
		printf("%d and %d\n", index_1, index_1);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	int		errs		= 0;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	int		fd_1		= 0;
	int		fd_2		= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty			= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)
	{
		gsc_label_init(38);
		test	= time(NULL);
		errs	= _parse_args(argc, argv);

		if (errs)
			break;

		// Introduction

		t_limit	= exec + (_minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(_def_index_1, _def_index_2);
		errs	+= dio32_init_util(1);
		errs	+= os_id_driver(dio32_open, dio32_read, dio32_close);
		errs	+= dio32_count_boards(1, &qty);

		if ((errs) || (qty <= 0))
			break;

		// Open access to devices

		errs	+= dio32_open_util(_def_index_1, 0, _def_index_1, 1, &fd_1);

		if (_def_index_1 != _def_index_2)
			errs	+= dio32_open_util(_def_index_2, 0, _def_index_2, 1, &fd_2);
		else
			fd_2	= fd_1;

		if (errs == 0)
		{
			// Device identification

			errs	= dio32_id_device(fd_1, _def_index_1, 1);

			if (_def_index_1 != _def_index_2)
				errs	+= dio32_id_device(fd_2, _def_index_2, 1);

			// Perform testing

			errs	+= perform_tests(_def_index_1, fd_1, _def_index_2, fd_2);

			// Close device access

			errs	+= dio32_close_util(fd_1, _def_index_1, 1);

			if (_def_index_1 != _def_index_2)
				errs	+= dio32_close_util(fd_2, _def_index_2, 1);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if ((_test_limit > 0) && (_test_limit <= tests))
			break;

		if (_continuous == 0)
			break;

		if ((_ignore_errors == 0) && (errs))
			break;

		if ((_minute_limit) && (now >= t_limit))
			break;
	}

	ret	= ((qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


