// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/DIO32/DIO32_Linux_1.x_GSC_DN/samples/dout/perform.c $
// $Rev: 51804 $
// $Date: 2022-10-14 12:33:07 -0500 (Fri, 14 Oct 2022) $

// DIO32: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#ifdef CLOCK_MONOTONIC_RAW
#define	CLOCK		CLOCK_MONOTONIC_RAW
#else
#ifdef CLOCK_MONOTONIC
#define	CLOCK		CLOCK_MONOTONIC
#else
#define	CLOCK		CLOCK_REALTIME
#endif
#endif



//*****************************************************************************
#ifndef	CLOCK_MONOTONIC_RAW
#define	clock_gettime(c,ts)	_my_clock_gettime(ts)

static int _my_clock_gettime(struct timespec* ts)
{
	int				ret;
	struct timeval	tv;

	if (ts)
	{
		ret	= 0;
		gettimeofday(&tv, NULL);
		ts->tv_sec	= tv.tv_sec;
		ts->tv_nsec	= tv.tv_usec * 1000;
	}
	else
	{
		ret	= 1;
	}

	return(ret);
}
#endif



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= dio32_irq_hl_enable		(args->fd, -1, 1, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
	errs	+= dio32_irq_lh_enable		(args->fd, -1, 1, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);
	errs	+= dio32_io_out_clock		(args->fd, -1, 1, -1, DIO32_ACT_TX, 0x11111111, 0, NULL);
	errs	+= dio32_io_out_dir			(args->fd, -1, 1, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0xFFFFFFFF, NULL);
	errs	+= dio32_io_out_tri_state	(args->fd, -1, 1, -1, DIO32_ACT_TX, 0xFFFFFFFF, 0, NULL);

	gsc_label_level_dec();
	return(errs);
}



//****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("    Seq   UU Byte   UL Byte   LU Byte   LL Byte \n");
	printf("    No.   76543210  76543210  76543210  76543210  Time Stamp      Delta         \n");
	printf("    ====  ========  ========  ========  ========  ==============  ==============\n");
}



//*****************************************************************************
static int _dev_dio_out(const args_t* args, u32 value)
{
	static	int				count	= 0;
	static	os_time_ns_t	t0;
	static	os_time_ns_t	ts;
	static	os_time_ns_t	ts_last;

	u32						bit;
	int						bits	= 32;
	long long				delta;
	int						errs	= 0;
	int						i;

	if (count == 0)
	{
		clock_gettime(CLOCK, &t0);
		clock_gettime(CLOCK, &ts);
	}

	ts_last	= ts;
	os_time_get_ns(&ts);
	errs	+= dio32_io_out_val(args->fd, -1, 0, -1, DIO32_ACT_TX, 0xFFFFFFFF, value, NULL);

	count++;
	printf("    %-4d  ", count);

	for (i = bits - 1; i >= 0; i--)
	{
		bit	= 0x1 << i;

		if (bit & 0x00808080)
			printf("  ");

		printf("%s", (value & bit) ? "1" : "0");
	}

	if (count == 1)
	{
		t0		= ts;
		ts_last	= ts;
	}

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * t0.tv_sec + t0.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	printf("\n");
	fflush(stdout);

	if (args->ms_wait)
		os_sleep_ms(args->ms_wait);

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	char	buf[80];
	int		errs;
	int		i;
	int		r;

	errs	= _config(args);

	for (r = 1; errs == 0; r++)
	{
		if (args->repeat)
		{
			if (r > args->repeat)
				break;

			printf("\n");
			sprintf(buf, "Output Pattern #%d of %d", r, args->repeat);
			gsc_label(buf);
			printf("\n");
		}
		else
		{
			if (r > 1)
				break;
		}


		_output_header();

		gsc_label_level_inc();

		errs	+= _dev_dio_out(args, 0x00000000);
		errs	+= _dev_dio_out(args, 0xFFFFFFFF);
		errs	+= _dev_dio_out(args, 0x00000000);
		errs	+= _dev_dio_out(args, 0xFFFFFFFF);
		errs	+= _dev_dio_out(args, 0x00000000);

		// Walking "1"

		for (i = 0; i <= 31; i++)
			errs	+= _dev_dio_out(args, 0x1 << i);

		errs	+= _dev_dio_out(args, 0x00000000);

		// Walking "0"

		for (i = 0; i <= 31; i++)
			errs	+= _dev_dio_out(args, ~(0x1 << i));

		errs	+= _dev_dio_out(args, 0x00000000);
		errs	+= _dev_dio_out(args, 0xFFFFFFFF);
		errs	+= _dev_dio_out(args, 0x00000000);
		errs	+= _dev_dio_out(args, 0xFFFFFFFF);
		errs	+= _dev_dio_out(args, 0x00000000);

		gsc_label_level_dec();
	}

	printf("\n");
	gsc_label("Status");
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	gsc_label_level_dec();
	return(errs);
}



